'' =================================================================================================
''
''   File....... jm_servo_in.spin
''   Purpose.... Measure servo pulse width in microseconds or system ticks
''               -- blocking code; make sure that servo signal is active
''               -- always uses CTRA
''   Author..... Jon McPhalen
''               Copyright (c) 2025 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon.mcphalen@gmail.com
''   Started....
''   Updated.... 08 JAN 2025
''
'' =================================================================================================

con

  VERSION = 1_0_0


var

  long  pin

  long  us10
  long  us05


pub null

'' This is not an application


pub start(servopin)

'' Setup servo input object

  pin := servopin                                               ' copy pin

  dira[pin] := 0                                                ' float low
  ina[pin]  := 0

  us10 := clkfreq / 1_000_000                                   ' ticks in 1.0us
  us05 := us10 >> 1                                             ' ticks in 0.5us


pub active : result | t

'' Looks for servo pulse on defined input pin
'' -- looks for 0->1 edge(s)

  ctra := (%01010 << 26) | pin                                  ' set POSEDGE detect
  frqa := 1
  phsa := 0

  waitcnt(cnt + (clkfreq / 20))                                 ' wait 50ms

  result := (phsa > 0)                                          ' true if activity detected


pub usecs : result

'' Return servo pulse width in microseconds

  return (ticks + us05) / us10


pub ticks : result | mask

'' Return servo pulse width in system tics
'' -- servo pulse is 0->1->0
'' -- uses CTRA

  mask := |<pin                                                 ' create mask for pin

  dira[pin] := 0
  ctra := (%01000 << 26) | pin
  frqa := 1

  waitpne(mask, mask, 0)                                        ' wait for low
  phsa := 0                                                     ' reset measurement
  waitpeq(mask, mask, 0)                                        ' wait for high
  waitpne(mask, mask, 0)                                        ' wait for low

  result := phsa


con { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}